"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.load = void 0;
const fs = require("fs");
const typedoc_1 = require("typedoc");
const converter_1 = require("typedoc/dist/lib/converter");
const events_1 = require("typedoc/dist/lib/output/events");
require("./GithubPluginMonkeyPatch");
require("./handlebarsDebug");
function load({ application }) {
    if (!fs.existsSync('docgen.json')) {
        throw new Error(`${process.cwd()}/docgen.json doesn't exist`);
    }
    const navigation = JSON.parse(fs.readFileSync('docgen.json').toString()).navigation;
    if (!navigation || Object.keys(navigation).length === 0) {
        throw new Error(`${process.cwd()}/docgen.json doesn't contain a navigation object`);
    }
    application.converter.on(converter_1.Converter.EVENT_RESOLVE_END, (context) => {
        // Remove the "References" section (re-exports --  revisit this when Typedoc library mode lands in 0.18.x)
        for (const reflection of context.project.getReflectionsByKind(typedoc_1.ReflectionKind.Reference)) {
            context.project.removeReflection(reflection);
        }
        // Rename other included @uirouter modules to a nicer name, e.g., @uirouter/core/state/stateService
        for (const reflection of context.project.getReflectionsByKind(typedoc_1.ReflectionKind.Module)) {
            if ((reflection.sources || []).find((x) => x.fileName.includes('@uirouter'))) {
                const match = new RegExp('"?(node_modules/)?(@uirouter/[^/]+)(/lib|/src)?(/.*?)(.d)?"?$').exec(reflection.name);
                if (match) {
                    reflection.name = `${match[2]}${match[4]}`;
                }
            }
        }
        // Rename source file names of included @uirouter files, stripping of the prefixed /src
        Object.values(context.project.reflections).forEach((reflection) => {
            if (!reflection.kindOf(typedoc_1.ReflectionKind.Module)) {
                (reflection.sources || []).forEach((source) => {
                    source.fileName = source.fileName.replace(/^\/?(project\/)?(src\/includes\/)?/g, '');
                });
            }
        });
    });
    // Do not show navigation for every Module.
    // Instead, load navigation from docgen.json and manually construct the nav links
    application.renderer.on(events_1.PageEvent.BEGIN, (pageEvent) => {
        const reflection = pageEvent.model;
        let project = reflection;
        while (project && !(project instanceof typedoc_1.ProjectReflection)) {
            project = project.parent;
        }
        if (reflection instanceof typedoc_1.ContainerReflection) {
            // Build the bespoke navigation from docgen.js
            const parent = pageEvent.navigation;
            parent.children = [];
            Object.keys(navigation).forEach((sectionName) => {
                const section = new typedoc_1.NavigationItem(sectionName, null, parent);
                section.isVisible = true;
                navigation[sectionName].forEach((childName) => {
                    const child = Object.values(project.reflections).find((x) => x.name === childName);
                    if (child) {
                        const item = new typedoc_1.NavigationItem(child.name, child.url, parent, child.cssClasses, child);
                        item.isVisible = true;
                        item.isInPath = true;
                        item.isCurrent = true;
                    }
                });
            });
        }
    });
    application.renderer.on(events_1.RendererEvent.BEGIN, (rendererEvent) => {
        const project = rendererEvent.project;
        // Make all links to the "Globals" reflection point to the index.html
        // Effectively replaces the "Globals" page with the index page.
        function findGlobalReflection(ref) {
            if (!ref)
                return undefined;
            if (ref.kind === typedoc_1.ReflectionKind.Global)
                return ref;
            return findGlobalReflection(ref.parent);
        }
        // Why can't I find by ReflectionKind.Global?
        let externalModules = project.getReflectionsByKind(typedoc_1.ReflectionKind.Module);
        externalModules
            .map(findGlobalReflection)
            .filter((x) => !!x)
            .reduce((acc, item) => (acc.includes(item) ? acc : acc.concat(item)), [])
            .forEach((global) => (global.url = 'index.html'));
    });
}
exports.load = load;
